﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.IO;
using System.Threading;

namespace CommunityDetection
{
    class Program
    {

		public Program(string programName, string configFile)
		{
			

			ExpProgram program = null;

			//program
			if (programName != null)
			{
				try
				{
					Type type = Type.GetType(this.GetType().Namespace + "." + programName);
					if (type.IsSubclassOf(typeof(ExpProgram)))
						program = (ExpProgram)Activator.CreateInstance(type);
				}
				catch (Exception)
				{
					Console.WriteLine("Invalid program name commandline parameter: " + programName);
					return;
				}

			}
			else
			{
				//choose program
				List<ExpProgram> programs = new List<ExpProgram>();
				programs.Add(new GenerateCandidatesFile());
				programs.Add(new FilterCanidatesFile());
				programs.Add(new PrintSteps());
				programs.Add(new TestClassifier());
				programs.Add(new TestEMClassifier());
				programs.Add(new FinalProgram());
				programs.Add(new FinalProgramV2());

				Console.WriteLine("Choose which program you want to run:");
				while (program == null)
				{
					for (int i = 0; i < programs.Count; i++)
					{
						Console.WriteLine(" " + (i + 1) + ". " + programs[i].GetType().Name);
					}
					Console.WriteLine(" X. Exit program");
					try
					{
						string line = Console.ReadLine();
						if (line.Length == 1 && line.ToLower()[0] == 'x')
						{
							Environment.Exit(0);
						}
						program = programs[int.Parse(line) - 1];

					}
					catch (Exception)
					{
						Console.WriteLine("Invalid option, type the number of the program you want to run:");
					}
				}
			}
			Console.WriteLine("Loading "+program+".. Done!");

			string[] args = null;
			
			//loading config file
			if (configFile != null) //commandline parameter
			{
				args = readConfigFile(configFile);
				if (args == null)
				{
					Console.WriteLine("Invalid configuration file commandline parameter: " + configFile);
					return;
				}

			}
			else
			{
				bool chooseFile = true;
				if (File.Exists("config.cfg"))
				{
					Console.WriteLine("Load config.cfg file? (y/n)");
					while (true)
					{
						string line = Console.ReadLine();
						if (line.Length != 1 || (line.ToLower()[0] != 'n' && line.ToLower()[0] != 'y'))
						{
							Console.WriteLine("Invalid option, load config.cfg? (y/n)");
							continue;
						}

						if (line.ToLower()[0] == 'y') //load config.cfg
						{
							Console.Write("Loading config.cfg.. ");
							args = readConfigFile("config.cfg");
							Console.WriteLine("Done!");
							chooseFile = false;
						}
						break;
					}
				}

				if(chooseFile)
				{
					string file = null;
					Console.WriteLine("Type the path to the configuration file:");
					while (file == null)
					{
						string line = Console.ReadLine();
						if (!File.Exists(line))
							Console.WriteLine("File " + line + " does not exist. Enter the path to the configuration file:");
						else
							file = line;
						
					}
					args = readConfigFile(file);
				}
			}
			Console.WriteLine("==== begin config file arguments ====");
			foreach (string s in args)
				Console.WriteLine(s);
			Console.WriteLine("==== end config file arguments ====");

			//program arguments
			if (args == null || args.Length < 3)
			{
				Console.WriteLine("Error reading config file");
				return;
			}
			Console.Write("Loading program arguments.. ");
			try
			{
				string[] programargs = new string[args.Length - 3];
				for (int i = 3; i < args.Length; i++)
					programargs[i-3] = args[i];
				program.loadBasicInfo(args[0], args[1], args[2]);
				program.loadArguments(programargs);
				Console.WriteLine("Done!");
			}
			catch(Exception e)
			{
				Console.WriteLine("Failed: invalid argument number or format: ");
				Console.WriteLine();
				Console.WriteLine(e);
				Console.WriteLine();
				throw e;
			}

			//creating logfile
			Console.WriteLine("Initializing program...");
			try
			{
				program.init();
				Console.WriteLine("Program finished initializing");
				
			}
			catch (Exception e)
			{
				Console.WriteLine("Failed: ");
				Console.WriteLine();
				Console.WriteLine(e);
				Console.WriteLine();
				throw e;
			}

			Console.WriteLine("Start running program at " + DateTime.Now.ToString("dd-MM-yyyy HH:mm"));
			TimeMeasure timer = new TimeMeasure();
			try
			{
				program.start();
				Console.WriteLine("Finished running program in " + timer + " secondes.");
			}
			catch (Exception e)
			{
				Console.WriteLine("Failed: ");
				Console.WriteLine();
				Console.WriteLine(e);
				Console.WriteLine();
				throw e;

			}
			Console.Write("Closing logfile.. ");
			program.exit();
			Console.WriteLine("Done!");
			return;
		}

	
		public string[] readConfigFile(string file)
		{
			try
			{
				StreamReader reader = new StreamReader(file);
				string line = null;
				List<string> args = new List<string>();
				while(null != (line = reader.ReadLine()))
				{
					if (line.Equals("") || line[0] == '#')
						continue;
					args.Add(line.Trim());
				}
				return args.ToArray();
			}
			catch (Exception e)
			{
				Console.WriteLine(e);
				return null;
			}
		}

		static void Main(string[] args)
        {
			//args = new string[] { "FinalProgramV2", "D:/Projecten/Afstuderen/svn/program/config.cfg" };
			//args = new string[] { "FinalProgramV2", "C:/Projects/icc/program/config.cfg" };
			Console.WriteLine("Welcome, starting program... Done!");
			new Program(args != null && args.Length>0 ? args[0]:null,args != null && args.Length>1 ? args[1]:null);
			Console.WriteLine("Press key to exit");
            Console.ReadKey();
        }
		
    }
}
