﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	class DescriptionCandidateGenerator : CandidatesGenerator
	{
		private List<HashSet<int>[]> selectors;

		public DescriptionCandidateGenerator(string description)
		{

			//char array
			char[] ch = description.Replace('v', '|').ToCharArray();

			//last seen character
			char last = ' ';

			//only keep relevant numbers and remove additional spaces
			List<char> filtered = new List<char>();
			for (int i = 0; i < ch.Length; i++)
			{
				if (!" 0123456789&|!".Contains(ch[i]))
					ch[i] = ' ';

				if (ch[i] == ' ' && last == ' ')
					continue;
				last = ch[i];
				filtered.Add(ch[i]);
			}
			//rebuild string
			description = new string(filtered.ToArray()).Trim();

			//split on OR
			string[] patterns = description.Split('|');

			//list of conjunction
			selectors = new List<HashSet<int>[]>();
			foreach (string p in patterns) // p1 OR p2 OR p..
			{
				HashSet<int>[] selector = new HashSet<int>[2];
				selector[0] = new HashSet<int>(); //negative items in this conjunction
				selector[1] = new HashSet<int>(); //positive items in this conjunction
				string[] parts = p.Split('&'); //split on and
				foreach (string part in parts)
				{
					int set = part.Contains('!') ? 0 : 1;
					selector[set].Add(int.Parse(part.Replace('!', ' ').Trim())); //add to set 
				}
				selectors.Add(selector);
			}
		}

		public override ThreadSafeQueue<Community> generate(UndirectedGraph graph, BinaryMatrix dm)
		{
			ThreadSafeQueue<Community> queue = new ThreadSafeQueue<Community>();
			Community c = new Community(graph);
			for (int i = 0; i < dm.M; i++)
			{
				BitArray ba = dm.getRow(i);
				foreach (HashSet<int>[] selector in selectors)
				{
					//check of all the items in selector[0] are not present, if so continue
					bool doContinue = false;
					foreach(int j in selector[0])
					{
						if (ba.get(j))
						{
							doContinue = true;
							break;
						}
					}
					if (doContinue)
						continue;

					//check of all the items in selector[1] are not present, if so continue
					doContinue = false;
					foreach (int j in selector[1])
					{
						if (!ba.get(j))
						{
							doContinue = true;
							break;
						}
					}
					if (doContinue)
						continue;

					//both conditions are right for this pattern, so it is in the community go to next row
					c.AddVertice(i);
					break;
				}
			}

			queue.Enqueue(c);
			return queue;
		}

		protected override ThreadSafeQueue<Community> generate(ThreadSafeQueue<Community> queue)
		{
			throw new NotImplementedException();
		}

		public override string abbr()
		{
			return "pgr";
		}

		public string Description
 		{
			get
			{
				List<string> parts = new List<string>();
				foreach (HashSet<int>[] s in selectors)
				{
					List<string> conjunction = new List<string>(s[0].Select(x => "!" + x));
					conjunction.AddRange(s[1].Select(x => "" + x));
					parts.Add("(" + string.Join(" & ", conjunction) + ")");
				}
				return string.Join(" | ", parts);
			}
		}
	}
}
