﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class BeamClassifier : Classifier
	{
		public BeamNode root;

		public BeamClassifier()
		{
			root = new BeamNode();
		}

		public override bool classify(BitArray values, int row)
		{
			return root.classify(values);
		}

		public List<KeyValuePair<BeamPattern, double[]>> getPatterns()
		{
			Queue<BeamNode> queue = new Queue<BeamNode>();
			queue.Enqueue(root);
			Dictionary<BeamPattern, double[]> map = new Dictionary<BeamPattern, double[]>();
			double totalnt = root.nt;
			while (queue.Count > 0)
			{
				BeamNode node = queue.Dequeue();
				if (node.splitPattern == null)
					continue;
				queue.Enqueue(node.left);
				queue.Enqueue(node.right);
				if (!map.ContainsKey(node.splitPattern))
					map[node.splitPattern] = new double[3];
				map[node.splitPattern][0] += node.right.nt+node.right.nf; //size of partitions containing splitpattern
				map[node.splitPattern][1] += node.right.nt; //number of in_comm containing splitpattern
				map[node.splitPattern][2] = (map[node.splitPattern][1]/totalnt) * (map[node.splitPattern][1]/map[node.splitPattern][0]); 
			}
			//StreamWriter writer = new StreamWriter("d:/patterns.txt");
			
	
			List<KeyValuePair<BeamPattern, double[]>> list = new List<KeyValuePair<BeamPattern, double[]>>(map);
			list = list.OrderByDescending(x => x.Value[2]).ToList();
			return list;

			//writer.Close();
		}

		public string doiets()
		{
			//external
			List<BeamNode> external = new List<BeamNode>();
			Queue<BeamNode> nodes = new Queue<BeamNode>();
			nodes.Enqueue(root);
			while (nodes.Count > 0)
			{
				BeamNode n = nodes.Dequeue();
				if (n.left == null && n.right == null)
				{
					external.Add(n);
				}
				else
				{
					nodes.Enqueue(n.left);
					nodes.Enqueue(n.right);
				}
			}
			StringBuilder sb = new StringBuilder();
			foreach (BeamNode b in external)
			{
				if (!b.classlabel)
					continue;
				string pattern = "";
				BeamNode current = b;
				while (true)
				{
					if (current.parent == null)
						break;
					if (current.parent.left == current)
						pattern += " & !" + current.parent.splitPattern.ToString();
					else
						pattern += " & " + current.parent.splitPattern.ToString();
					current = current.parent;
				}
				sb.AppendLine(pattern);
			}
			return sb.ToString();
		}

		public NBClassifier rebuild()
		{
			//external
			List<BeamNode> external = new List<BeamNode>();
			Queue<BeamNode> nodes = new Queue<BeamNode>();
			nodes.Enqueue(root);
			while (nodes.Count > 0)
			{
				BeamNode n = nodes.Dequeue();
				if (n.left == null && n.right == null)
				{
					external.Add(n);
				}
				else
				{
					nodes.Enqueue(n.left);
					nodes.Enqueue(n.right);
				}
			}
			List<HashSet<int>> pinList;
			List<List<HashSet<int>>> poutList;
			pinList = new List<HashSet<int>>();
			poutList = new List<List<HashSet<int>>>();

			foreach (BeamNode b in external)
			{
				HashSet<int> pin;
				pin = new HashSet<int>();
				List<HashSet<int>> pout = new List<HashSet<int>>();
				if (!b.classlabel)
					continue;
				BeamNode current = b;
				while (true)
				{
					if (current.parent == null)
						break;
					BeamPattern p = current.parent.splitPattern;
					if (current.parent.left == current)
					{
						pout.Add(p.pattern);
					}
					else
					{
						foreach (int v in p.pattern)
							pin.Add(v);
					}
					current = current.parent;
				}
				pinList.Add(pin);
				poutList.Add(pout);
			}
			return new NBClassifier(pinList, poutList);
		}

		public void print(TextWriter writer)
		{
			root.print(0, writer);
		}
	}
}
