﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class CeapBuilder : Builder
	{
		Dictionary<ItemSet, int> p, n;
		Dictionary<ItemSet, double> growRate;

		HashSet<ItemSet> all;
		int sizeN, sizeP;
		Dictionary<ItemSet, double> chances;

		public CeapBuilder(BinaryMatrix data, string datadir) : base(data)
		{
			Console.Write("reading in_comm itemsets... ");
			p = readItemSets(datadir + "icc/comm1_in_3.isc");
			Console.WriteLine("Done "+p.Count);
			
			Console.Write("reading out_comm itemsets... ");
			n = readItemSets(datadir + "icc/comm1_out_3.isc");
			Console.WriteLine("Done "+n.Count);

			Console.Write("Creating hashset with al itemsets... ");
			all = new HashSet<ItemSet>();
			foreach (ItemSet s in p.Keys)
				all.Add(s);
			foreach (ItemSet s in n.Keys)
				all.Add(s);
			Console.WriteLine("Done " + all.Count);

			sizeN = 24648;
			sizeP = 352;

			Console.Write("Calculating grow rates... ");
			calcGrowRate();
			Console.WriteLine("Done " + growRate.Count);



		}


		public void calcGrowRate()
		{
			growRate = new Dictionary<ItemSet, double>();
			chances = new Dictionary<ItemSet, double>();
			int nn = 0; 
			
			foreach (ItemSet set in all)
			{
				int countP=0;
				int countN=0;
				if (p.ContainsKey(set))
				{
					countP = p[set];
				}
				if (n.ContainsKey(set))
				{
					countN = n[set];
				}


				double gr = 0;


				double supP = (double) countP / sizeP;
				double supN = (double) countN / sizeN;
				gr = supP / supN;
				if (countN == 0)
				{
					if (countP == 0)
						gr = 0;
					else
						gr = 0;//sizeN;
				}

				if (gr > 1.5)
				{
					growRate[set] = gr;
					chances[set] = (double)countP / (double) (countP + countN);
					Console.WriteLine(chances[set]);
				}
			}
			Console.WriteLine("N: " + nn);

		}

		public Dictionary<ItemSet, int> readItemSets(string file)
		{
			string line;
			int support;
			ItemSet itemset;

			Dictionary<ItemSet, int> set = new Dictionary<ItemSet, int>();

			//in
			StreamReader reader = new StreamReader(file);
			reader.ReadLine();
			reader.ReadLine();
			while (null != (line = reader.ReadLine()))
			{
				if (line == "")
					continue;
				string[] parts = line.Split(':');
				parts = parts[1].Split('(');
				support = int.Parse(parts[1].Substring(0, parts[1].Length - 1));
				itemset = new ItemSet(support);
				parts = parts[0].Trim().Split(' ');
				foreach (string s in parts)
				{
					itemset.Add(int.Parse(s));
				}
				set[itemset] = itemset.count;
			}
			return set;
		}

		public override Classifier build(HashSet<int> trueRows)
		{
			throw new NotImplementedException();
		}
	}
}
