﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

/**
 * Note: The alpha parameter was only used during the research to balance between
 * precision and recall. Now it just uses the f1 score, alpha is not used anymore
 */
namespace CommunityDetection
{
	public class ConjunctionBuilder : Builder
	{
		double alpha;
		int maxPatternLength;

		public ConjunctionBuilder(BinaryMatrix data, int maxPatternLength, double alpha) : base(data)
		{
			this.alpha = alpha;
			this.maxPatternLength = maxPatternLength;
		}

		public double measure(double tp, double fp, double tn, double fn)
		{
			double p = tp / (tp + fp);
			double r = tp / (tp + fn);
			double v = (2.0 * p * r) / (p+r);
			return v;
		}

		public override Classifier build(HashSet<int> trueRows)
		{

			//create current community and attribute set
			HashSet<int> current = new HashSet<int>();
			HashSet<int> attributes = new HashSet<int>();
			for (int a = 0; a < data.N; a++)
				attributes.Add(a);

			//create sets for classifier
			HashSet<int> inA, outA;
			inA = new HashSet<int>();
			outA = new HashSet<int>();

			//determine start set of classifier
			for (int a = 0; a < data.N; a++)
			{
				bool allT = true;
				bool allF = true;

				foreach (int r in trueRows)
				{
					if (data.get(r, a))
						allF = false;
					else
						allT = false;

					if (!allF && !allT)
						break;
				}

				//if(allT)
				//	inA.Add(a);
				//if (allF)
				//	outA.Add(a);
			}

			//update attribute set
			foreach (int a in inA.Union(outA))
				attributes.Remove(a);
			
			//create current community
			for (int r = 0; r < data.M; r++)
			{
				bool inCommunity = true;
				
				//contains all inA atreributes?
				foreach (int a in inA)
				{
					if (!data.get(r, a))
					{
						inCommunity = false;
						break;
					}
				}

				if (!inCommunity)
					continue;

				//doest not contain outA attributes?
				foreach (int a in outA)
				{
					if (data.get(r, a))
					{
						inCommunity = false;
						break;
					}
				}

				if (inCommunity)
					current.Add(r);
			}
			
			double best = getQuality(trueRows, current);
			double bestDelta = 0;
			int bestAttribute = -1;
			bool bestAdd = false;

			for (int i = 0; i < maxPatternLength; i++)
			{
				foreach (int a in attributes)
				{
					//add
					double qAdd = getQuality(trueRows, current, a, true);
					if (qAdd - best > bestDelta)
					{
						bestDelta = qAdd - best;
						bestAttribute = a;
						bestAdd = true;
					}

					//remove
					qAdd = getQuality(trueRows, current, a, false);
					if (qAdd - best > bestDelta)
					{
						bestDelta = qAdd - best;
						bestAttribute = a;
						bestAdd = false;
					}
				}

				if (bestDelta > 0)
				{
					List<int> remove = new List<int>();
					if (bestAdd)
					{
						inA.Add(bestAttribute);
						foreach (int r in current)
						{
							if (!data.get(r, bestAttribute))
								remove.Add(r);
						}
					}
					else
					{
						outA.Add(bestAttribute);
						foreach (int r in current)
						{
							if (data.get(r, bestAttribute))
								remove.Add(r);
						}
					}
					foreach (int r in remove)
						current.Remove(r);
					best = getQuality(trueRows, current);
					bestDelta = 0;
					bestAttribute = -1;
				}
				else
					break;
			}

			return new ConjunctionClassifier(inA,outA);
		}

		public double getQuality(HashSet<int> trueRows, HashSet<int> current, int a, bool contains)
		{
			HashSet<int> newCurrent = new HashSet<int>();
			foreach (int r in current)
			{
				if (data.get(r, a) == contains)
					newCurrent.Add(r);
			}
			return getQuality(trueRows, newCurrent);
		}

		public double getQuality(HashSet<int> trueRows, HashSet<int> current)
		{
			int tp = trueRows.Intersect(current).Count();
			int fp = current.Count - tp;
			int tn = data.M - trueRows.Union(current).Count();
			int fn = trueRows.Count - tp;
			return measure(tp, fp, tn, fn);
		}
	}
}
