﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{

	class DecisionTreeBuilder : Builder
	{
		public ImpurityMeasure measure;
		
		public int[] columnSum;
		public List<int> attributeList;
		public HashSet<int> rowSet;

		public DecisionTreeBuilder(BinaryMatrix data, ImpurityMeasure measure): base(data)
		{
			this.measure = measure;
			
			columnSum = data.columnSum();
			
			attributeList = new List<int>(data.N);
			for (int i = 0; i < data.N; i++)
				attributeList.Add(i);

			rowSet = new HashSet<int>();
			for (int i = 0; i < data.M; i++)
				rowSet.Add(i);
		}


		public ImpurityMeasure getMeasure()
		{
			return measure;
		}

		public DecisionTree buildTree(HashSet<int> trueRows, HashSet<int> rows)
		{
			DecisionTree tree = new DecisionTree(this);
			if (trueRows.IsProperSubsetOf(rows))
			{
				tree.create(trueRows, new HashSet<int>(rows));
			}
			else
			{
				tree.create(new HashSet<int>(trueRows.Intersect(rows)), new HashSet<int>(rows));
			}
			return tree;
		}

		public override Classifier build(HashSet<int> trueRows)
		{
			return buildTree(trueRows, rowSet);
		}


		public List<KeyValuePair<double,DecisionTree>> getTreeSequence(DecisionTree tree)
		{
			List<KeyValuePair<double, DecisionTree>> list = new List<KeyValuePair<double, DecisionTree>>();
			KeyValuePair<double, DecisionTree> kv = new KeyValuePair<double, DecisionTree>(0, tree);
			list.Add(kv);

			kv = kv.Value.prune();
			while (kv.Value != null)
			{
				list.Add(kv);
				kv = kv.Value.prune();
			}
			return list;
		}
	}
}
