﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	public class EMBuilder : Builder
	{
		int[] sum;
		
		public EMBuilder(BinaryMatrix data): base(data)
		{
			sum = new int[data.N];
			for (int i = 0; i < data.M; i++)
			{
				for (int j = 0; j < data.N; j++)
				{
					if(data.get(i,j))
						sum[j]++;
				}
			}
		}

		public override Classifier build(HashSet<int> trueRows)
		{
			int sizeIn = trueRows.Count;
			int sizeOut = data.M - sizeIn;
			int[] sumIn = new int[data.N];
			int[] sumOut = new int[data.N];
			double[] probIn = new double[data.N];
			double[] probOut = new double[data.N];
			
			foreach (int r in trueRows)
			{
				for (int a = 0; a < data.N; a++)
				{
					if (data.get(r, a))
						sumIn[a]++;
				}
			} 

			for (int a = 0; a < data.N; a++)
			{
				sumOut[a] = sum[a] - sumIn[a];
				probIn[a] = (double) sumIn[a] / (double) sizeIn;
				probOut[a] = (double) sumOut[a] / (double) sizeOut;
				if (probIn[a] <= 0)
					probIn[a] = 0.00001;
				if (probOut[a] <= 0)
					probOut[a] = 0.0001;
				
				if (probIn[a] >= 1)
					probIn[a] = 0.9999;
				if (probOut[a] >= 1)
					probOut[a] = 0.9999;
			}



			return new EMClassifier(probIn,probOut);
		}
	}
}
