﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class FPTree
	{
		public FPNode root;

		//header table
		public List<FPNode> header;

		//itemnumber to index in header
		public Dictionary<int, int> headerIndex;

		//number of nodes in tree
		public int count;

		public FPTree()
		{
			root = new FPNode();
			count = 1;
			root.item = -1;
			headerIndex = new Dictionary<int, int>();
			header = new List<FPNode>();
		}

		/**
		 * Insert a transaction in an FP tree
		 * @param t Transaction orderd by frequency in conditional database
		 */
		public void insertTree(List<int> t)
		{
			//start at root
			FPNode current = root;

			for (int i = 0; i < t.Count; i++)
			{
				//get item
				int item = t[i];

				//look if the is a child of current node with the same itemnumber as t[i]
				FPNode child = current.getChild(item);

				if (child == null) //if not exist create node
				{
					//create
					child = new FPNode();
					count++;

					//store information
					child.freq = 1;
					child.item = item;

					//create tree information
					child.parent = current;
					current.AddChild(child);

					//update header
					if(headerIndex.ContainsKey(item)) //item seen before, add pointer from last seen to new one
					{
						//find last one
						FPNode end = header[headerIndex[item]];
						while(end.link != null)
							end = end.link;

						//add pointer
						end.link = child;
					}
					else //not seen before, add to header table
					{
						int index = header.Count; //position in index table
						header.Add(child);
						headerIndex[item] = index; //save index
					}
				}
				else //exist, just increment frequency
				{
					child.freq++;
				}

				//go to next one
				current = child;
			}
		}


		/**
		 * Print the FPTree on a readable way
		 */
		public void print()
		{
			print(Console.Out);
		}


		public void print(TextWriter writer)
		{
			writer.WriteLine("## FPTRee ##");
			writer.WriteLine(" header:");
			int i=0;
			foreach(FPNode n in header)
			{
				writer.WriteLine("  " + (i++) + ": node " + n.id+" (item: "+n.item+")");
			}
			writer.WriteLine(" tree:");
			root.print(2, writer);
		}

	}
}
