﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	class ResultSet
	{
		HashSet<Community> set;
		UndirectedGraph graph;

		public ResultSet(UndirectedGraph graph)
		{
			this.graph = graph;
			set = new HashSet<Community>(new CommunityEqualityComparer());
		}

		public void Add(Community c)
		{
			lock (set)
			{
				set.Add(c);
			}
		}

		public bool Contains(Community c)
		{
			lock (set)
			{
				return set.Contains(c);
			}
		}

		public int Count
		{
			get
			{
				lock (set)
				{
					return set.Count;
				}
			}
		}

		public int getNumberEquations()
		{
			lock(set)
			{
				CommunityEqualityComparer comp = (CommunityEqualityComparer) set.Comparer;
				return comp.count;
			}
		}

		public List<Community> toSortedList()
		{
			List<Community> list;
			lock (set)
			{
				list = new List<Community>(set.Count);
				foreach (Community c in set)
				{
					list.Add(c);
				}
			}
			list.Sort();
			return list;
		}

		public double[] avgJaccardDistance(int k)
		{
			List<Community> list = toSortedList();
			k = Math.Min(list.Count, k);
			BitArray[] ba = new BitArray[k];

			List<double> jd = new List<double>();
			double cg = 0;
			double csize=0;
			for (int i = 0; i < k; i++)
			{
				ba[i] = list[i].getVerticeMembership();
				for (int j = 0; j < i; j++)
				{
					jd.Add(ba[i].JaccardDistance(ba[j]));
				}
				csize += list[i].Size;
				cg += list[i].communityGain();
				
			}
			double min=double.MaxValue;
			double max = double.MinValue;
			double avg = 0;
			foreach (double d in jd)
			{
				avg += d;
				min = Math.Min(min, d);
				max = Math.Max(max, d);
			}
			avg /= jd.Count;
			double stdev = 0;
			foreach (double d in jd)
			{
				stdev += (d - avg)*(d - avg);
			}
			stdev = Math.Sqrt(stdev / jd.Count);
			return new double[]{avg,min,max,stdev,csize/k,cg/k};
		}

		public double entropy(int k)
		{
			List<Community> list = toSortedList();
			k = Math.Min(list.Count, k);
			double[] prob = new double[graph.Count];
			double[] e = new double[k];
			
			//count 1's in membership lists
			for (int i = 0; i < k; i++)
			{
				foreach(int v in list[i].getVertices().Keys)
				{
					prob[v]++;
				}
			}

			//divide by k
			for (int i = 0; i < graph.Count; i++)
			{
				prob[i] /= (double) k;
			} 


			//calculate entropy for individuals
			for (int i = 0; i < k; i++)
			{
				for (int v = 0; v < graph.Count; v++)
				{
					double p;
					if (list[i].ContainsVertice(v))
						p = prob[v];
					else
						p = 1 - prob[v];
					if(p > 0)
						e[i] += p * Math.Log(p, 2);
				}
				e[i] *= -1;
				Console.WriteLine(e[i]);
			}

			return 0;
		}


		public double entropy(double topK)
		{
			return entropy((int) Math.Min(set.Count, (topK * set.Count)));

		}
	}
}
