﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public abstract class BinaryMatrix : BitClass
	{
		protected int m;

		public bool this[int i, int j]
		{
			get
			{
				return get(i, j);
			}
			set
			{
				set(i, j, value);
			}
		}

		public abstract bool get(int i, int j);
		public abstract void set(int i, int j, bool v);
		public abstract BitArray getRow(int i);
		public abstract int countValue(bool v);

		public List<int> getIndexesByValuePerColumn(int j, bool v)
		{
			List<int> indexes = new List<int>();
			for (int i = 0; i < m; i++)
			{
				if (get(i, j) == v)
					indexes.Add(i);
			}
			return indexes;
		}

		public int countColumn(int j, bool v)
		{
			int count = 0;
			for (int i = 0; i < m; i++)
			{
				if (get(i, j) == v)
					count++;
			}
			return count;
		}

		public abstract int countRow(int i, bool v);
		public abstract List<int> getIndexesByValuePerRow(int i, bool v);
		public abstract int[] columnSum();

		protected abstract void print(TextWriter writer, int type);

		public void WriteArffFile(string file, string dbName)
		{
			StreamWriter writer = new StreamWriter(file);
			writer.WriteLine("@relation " + dbName);
			for (int i = 0; i < n; i++)
			{
				writer.WriteLine("@attribute tag" + i + " {0,1}");
			}
			writer.WriteLine("@data");
			for (int i = 0; i < m; i++)
			{
				List<string> values = new List<string>();
				for (int j = 0; j < n; j++)
				{
					if (get(i, j))
						values.Add(j + " 1");
				}
				writer.WriteLine("{" + string.Join(",", values.ToArray()) + "}");
			}
			writer.Close();
		}

		public void Write(string file, int b)
		{
			StreamWriter writer = new StreamWriter(file);
			print(writer,b);
			writer.Close();
		}

		public void Write(string file)
		{
			Write(file, 10);
		}

		public double Density
		{
			get
			{
				double total = (double) m * n;
				double ones = countValue(true);
				return ones / total;
			}
		}

		public int M
		{
			get
			{
				return m;
			}
		}

		public override string ToString()
		{
			return "BinaryMatrix: M: " + m + ", N: " + n + ", Size: " + (m * n);
		}

	}
}
