﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CommunityDetection
{
	public class CommunityGain : QualityMeasure
	{
		bool countWithinDouble;

		public CommunityGain(bool countBetweenDouble)
		{
			this.countWithinDouble = countBetweenDouble;
		}

		public double independentSize(Community c)
		{
			Dictionary<int, int> vertices = c.getVertices();
			Dictionary<int, int> neighbours = c.getNeighbours();

			int w = 0;
			int b = 0;
			foreach (int value in vertices.Values)
				w += value;
			foreach (int value in neighbours.Values)
				b += value;
			return vertices.Count()+w+b;
		}

		public double communitySize(Community c)
		{
			Dictionary<int, int> vertices = c.getVertices();
			Dictionary<int, int> neighbours = c.getNeighbours();

			//community situation
			int w = 0;
			int b = 0;
			foreach (KeyValuePair<int, int> kv in vertices) //type W errors
			{
				w += c.Size - kv.Value - 1;
			}

			foreach (KeyValuePair<int, int> kv in neighbours) //type B errors
			{
				b += kv.Value;
			}
			if (countWithinDouble)
				return c.Size + w + b;
			else
				return c.Size + w/2.0 + b;
		}

		public override double communityGain(Community c)
		{
			return independentSize(c) - communitySize(c);
		}

		public override double gain(Community c, int v, bool add)
		{
			double factor = countWithinDouble ? 4 : 3;
			if (add)
			{
				return (2 * (c.Size + 1)) - (factor * errw(c, v) + 2);
			}
			else
			{
				return (factor * errw(c, v) + 2) - (2 * (c.Size));
			}
		}
	}
}
