﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public class EmDmRun
	{
		string logdir;
		ThreadSafeQueue<Community> queue;
		BeamBuilder builder;
		UndirectedGraph graph;
		BinaryMatrix dm;
		int maxEmDmIterations;
		CommunityStampSet afterEMstampSet;
		FinalProgram fp;
		Dictionary<int, KeyValuePair<Community, NBClassifier>> classifiedCommunities;


		public EmDmRun(FinalProgram fp, BeamBuilder builder, ThreadSafeQueue<Community> queue, UndirectedGraph graph, BinaryMatrix dm, string logdir, int maxEmDmIterations, CommunityStampSet stampSet, Dictionary<int, KeyValuePair<Community, NBClassifier>> classifiedCommunities)
		{
			this.fp = fp;
			this.builder = builder;
			this.queue = queue;
			this.logdir = logdir;
			this.dm = dm;
			this.graph = graph;
			this.maxEmDmIterations = maxEmDmIterations;
			this.afterEMstampSet = stampSet;
			this.classifiedCommunities = classifiedCommunities;
		}

		public void explog(string msg)
		{
			fp.explog(msg);
		}

		public void run()
		{
			while (!queue.Empty())
			{
				try
				{
					Community c = queue.Dequeue();
					Console.WriteLine("Starting with new candidate, "+queue.Count+" candidates left");
					processCandidate(c);
				}
				catch (InvalidOperationException)
				{
					break;
				}
			}
		}

		public void writeCommunity(Community c, int iteration, string place, string nextAction, NBClassifier cfr)
		{
			explog(c.ID + ";" + iteration + ";" + place + ";" + c.Size + ";" + c.communityGain() + ";" + c.intraClusterDensity() + ";"+ (cfr == null?"-;-;-;-;-":cfr.getInformation())+ ";" + nextAction + ";" + c.ToString());
		}

		public void processCandidate(Community c)
		{
			StreamWriter writer = new StreamWriter(logdir + "/" + c.ID + ".csv");
			writer.WriteLine("Community: " + c.ID);
			writer.WriteLine();
			int i = 0;
			NBClassifier nbc = null;
			while (maxEmDmIterations == 0 || i<maxEmDmIterations)
			{
				writer.WriteLine("---------------");
				writer.WriteLine("Iteration:;" + i);
				
				//em
				HashSet<int> before = new HashSet<int>(c.getVertices().Keys);
				writer.WriteLine("Phase:;EM");
				writer.WriteLine("Steps:");
				writer.WriteLine("Step;Size;ICD;CG;Vertices");
				writer.WriteLine("0;"+c.Size + ";" + c.intraClusterDensity() + ";" + c.communityGain() + ";" + c.ToString());
				int steps = 1;
				while (c.improve())
				{
					writer.WriteLine(steps+";" + c.Size + ";" + c.intraClusterDensity() + ";" + c.communityGain() + ";" + c.ToString());
					steps++;
				}

				//write information
				if (afterEMstampSet.Contains(c))
				{
					Community oc = afterEMstampSet.getCommunity(c.Stamp);
					if (c.ID != oc.ID)
					{
						writeCommunity(c, i, "0. After EM", "Stop, Allready classified in community " + oc.ID, null);
						writer.WriteLine("Allready classified in community " + oc.ID);
					}
					else
					{
						writeCommunity(c, i, "0. After EM", "Stop, earlier classified in this community", null);
						writer.WriteLine("Allready classified in earlier in this community");
					}
					writer.Close();
					return;
				}
				else
				{
					writeCommunity(c, i, "0. After EM", "DM step",null);
				}
				afterEMstampSet.Add(c);
				writer.WriteLine();
				writer.WriteLine("Summary:");
				HashSet<int> between = new HashSet<int>(c.getVertices().Keys);
				HashSet<int> added = new HashSet<int>(between.Except(before));
				HashSet<int> removed = new HashSet<int>(before.Except(between));
				writer.WriteLine(";Added:;" + added.Count);
				writer.WriteLine(";Removed:;" + removed.Count);
				writer.WriteLine();
				writer.WriteLine("Community:");
				writer.WriteLine(";Size: " + c.Size);
				writer.WriteLine(";CG: " + c.communityGain());
				writer.WriteLine(";ICD: " + c.intraClusterDensity());


				//dm
				if (builder != null)
				{
					writer.WriteLine();
					writer.WriteLine();
					writer.WriteLine("Phase:;DM");
					BeamClassifier classifier = (BeamClassifier) builder.build(between);
					nbc = classifier.rebuild();

					writer.WriteLine();
					writer.WriteLine("Patterns:");
					writer.WriteLine(nbc);
					int[] qualityData = nbc.quality(dm, between);
					writer.WriteLine();
					writer.WriteLine("Summary:");
					writer.WriteLine(";Precision:;" + classifier.precision(qualityData));
					writer.WriteLine(";Recall:;" + classifier.recall(qualityData));
					writer.WriteLine(";TP:;" + qualityData[0]);
					writer.WriteLine(";FP:;" + qualityData[1]);
					writer.WriteLine(";TN:;" + qualityData[2]);
					writer.WriteLine(";FN:;" + qualityData[3]);
	
					for (int v = 0; v < graph.Count; v++)
					{
						if(classifier.classify(dm.getRow(v)))
							c.AddVertice(v);
						else
							c.RemoveVertice(v);
					}

					HashSet<int> after = new HashSet<int>(c.getVertices().Keys);
					added = new HashSet<int>(after.Except(between));
					removed = new HashSet<int>(between.Except(after));
					writer.WriteLine("Classifier:");
					writer.WriteLine(";Added: " + added.Count);
					writer.WriteLine(";Removed: " + removed.Count);
					writer.WriteLine();
					writer.WriteLine("Community:");
					writer.WriteLine(";Size: " + c.Size);
					writer.WriteLine(";CG: " + c.communityGain());
					writer.WriteLine(";ICD: " + c.intraClusterDensity());
					writeCommunity(c, i, "1.After DM", i<maxEmDmIterations || maxEmDmIterations==0?"EM step":"Stop, max EMDM iterations reached", nbc);
					lock (classifiedCommunities)
					{
						classifiedCommunities[c.ID] = new KeyValuePair<Community, NBClassifier>(c, nbc);
					}
				}
				writer.WriteLine();
				i++;
			}
			writer.Close();
			writeCommunity(c, i, "2. After procession candidates","Nothing, max EMDM iterations reached", nbc);
		}



	}

	
}
