﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace CommunityDetection
{
	public abstract class ExpProgram
	{
		protected string expdir, datadir, expName, graphName;
		protected UndirectedGraph graph;
		private StreamWriter logwriter;
		TimeMeasure measure;
		TimeMeasure msgMeasure;


		public void init()
		{
			if (!Directory.Exists(expdir + this.GetType().Name))
			{
				Directory.CreateDirectory(expdir + this.GetType().Name);
			}

			Console.Write("Reading graph... ");
			graph = new UndirectedGraphList(datadir + graphName);
			Console.WriteLine("Done!");

			expName = this.GetType().Name + "/" + "results_" + DateTime.Now.ToString("yyyy_MM_dd_HHmm_ss");
			logwriter = new StreamWriter(expdir + expName + ".exp");
			Console.WriteLine("Running experiment: " + expName);
			log("*** Log " + DateTime.Now.ToString("dd-MM-yyyy HH:mm:ss") + " *****");
			log("");
			log("Program: " + this.GetType().Name);
			log("Experiment: " + expName);
			log("Datadir: " + datadir);
			log("Expdir: " + expdir);
			log("Graphname :" + graphName);
			log("Graph: |V|=" + graph.Count + ", |E|=" + graph.NumEdges);
			log("");
			log("**** Program output ****");
			log("");
			measure = new TimeMeasure();
		}

		public void basisLogInfo()
		{
		}

		~ExpProgram()
		{
			if (logwriter != null)
			{
				try
				{
					logwriter.Close();
				}
				catch (Exception)
				{ 
				}
			}

		}

		public void loadBasicInfo(string datadir, string expdir, string graphName)
		{
			this.datadir = datadir;
			this.expdir = expdir;
			this.graphName = graphName;
		}

		public abstract void loadArguments(string[] args);

		public void startmsg(string msg)
		{
			msgMeasure = new TimeMeasure();
			Console.Write(msg + "... ");
		}
		public void endmsg()
		{
			Console.WriteLine("Done in "+msgMeasure.getDuration()+" seconds!");
		}
		public override string ToString()
		{
			return this.GetType().Name;
		}

		public string getExpName()
		{
			return expName;
		}

		public virtual void log(string msg)
		{
			lock (logwriter)
			{
				logwriter.WriteLine(msg);
				logwriter.Flush();
			}
		}

		public abstract void start();
		
		public void exit()
		{
			Console.WriteLine("Finished experiment: " + expName);
			log("");
			log("**** End program output ****");
			log("");
			log("Finished: "+DateTime.Now.ToString("dd-MM-yyyy HH:mm:ss"));
			log("Duration: "+measure+" seconds");
			logwriter.Close();
		}
	}
}
