﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;

namespace CommunityDetection
{
	class GenerateCandidatesFile : ExpProgram
	{
		//list of all situations seen till now
		CommunityStampSet stamps;

		//queue of all candidates to be processed
		ThreadSafeQueue<Community> candidates;

		//generator
		CandidatesGenerator generator;

		//list Candidates
		ThreadSafeQueue<Community> result;

		int numThreads;

		public GenerateCandidatesFile()
		{

		}

		public override void loadArguments(string[] args) 
		{
			numThreads = int.Parse(args[0]);
			generator = new SharedNeighboursCG(int.Parse(args[1]));
		}

		public override void start()
		{
			Community.measure = new CommunityGain(false);
			CandidatesGenerator.smartStart = true;
			Operation.addWithoutGain = true;

			//candidates
			Console.Write("Generating candidates with " + generator + "... ");
			candidates = generator.generate(graph, null);
			int numCandidates = candidates.Count;
			Console.WriteLine("Done..");

			//result set
			Console.Write("Creating result set... ");
			result = new ThreadSafeQueue<Community>();
			Console.WriteLine("Done!");

			//stamp set
			Console.Write("Creating stamp set... ");
			stamps = new CommunityStampSet();
			Console.WriteLine("Done!");

			//Printing numbers:
			Console.WriteLine("Numbers:");
			Console.WriteLine(" - Graph: " + graph);
			Console.WriteLine(" - Candidates: " + candidates.Count);
			Console.WriteLine(" - Threads: " + numThreads);
				
			//running emdm
			DateTime start = DateTime.Now;
			Console.WriteLine("Processing candidate"+(candidates.Count > 1 ? "s" : "")+" ... ");

			//creating threads
			List<Thread> threads = new List<Thread>();
			for (int i = 0; i < numThreads; i++)
			{
				Thread t = new Thread(this.run);
				threads.Add(t);
				t.Start();
			}

			////wait for all threads to be finished
			foreach (Thread t in threads)
			    t.Join();
			while (candidates.Count > 0)
				result.Enqueue(candidates.Dequeue());
			
			//total duration
			TimeSpan durationSpan = DateTime.Now - start;
			int duration = (durationSpan.Days * 24 * 3600 + durationSpan.Hours * 3600 + durationSpan.Minutes * 60 + durationSpan.Seconds);
			Console.WriteLine("Done in " + duration + " seconds!");

			log("Datadir: " + datadir);
			log("Expdir: " + expdir);
			log("Graph: " + graphName);
			log("Threads: " + numThreads);
			log("Generator: " + generator);
			log("Candidates: " + numCandidates);
			log("Result communities: " + result.Count);
			log("Duration: " + duration);
			log("Stamps: " + stamps.Count);

			//sort results
			Console.Write("Sorting and filtering resultset of "+result.Count+" communities... ");
			StreamWriter writer = new StreamWriter(expdir+getExpName()+".cf");
			List<Community> communities = new List<Community>(result.Count);
			while(!result.Empty())
			{
				Community c = result.Dequeue();
				communities.Add(c);
			}
			communities = new List<Community>(communities.OrderByDescending(c => c.communityGain()).ThenBy(c => c.Size));
			Console.WriteLine("Done!");
			Console.Write("Writing result file of " + communities.Count + " communities... ");
			foreach(Community c in communities)
			{
				List<int> nodes = new List<int>(c.getVertices().Keys);
				nodes.Sort();
				string line = "";
				foreach (int v in nodes)
				{
					line+= " "+v;
				}
				writer.WriteLine(line.Trim());
			}
			writer.Close();
			Console.WriteLine("Done!");
		}

		public void run()
		{
			try
			{
				while (!candidates.Empty())
				{
					Community c = candidates.Dequeue();
					if (stamps.Contains(c))
					{
						continue;
					}
					stamps.Add(c);
					if (em(c))
						result.Enqueue(c);
					if (candidates.Count % 25 == 0 && candidates.Count > 0)
						Console.WriteLine("  " + candidates.Count + " candidates left , stamps: " + stamps.Count);
				}
			}
			catch (InvalidOperationException)
			{
				//Console.WriteLine(e);
			}
		}

		public bool em(Community c)
		{
			while (c.improve())
			{
				//check if this situation is seen before
				if (stamps.Contains(c))
				{
					return false; //seen before, go on;
				}
				stamps.Add(c);
			}
			return true;
		}

	}
}
